@@ -20,10 +20,13 @@ import traceback
 import zmq
 
 from six import iteritems
+from ducktape.services.service import MultiRunServiceIdFactory, service_id_factory
+from ducktape.services.service_registry import ServiceRegistry
 
 from ducktape.tests.event import ClientEventFactory
 from ducktape.tests.loader import TestLoader
 from ducktape.tests.serde import SerDe
+from ducktape.tests.status import FLAKY
 from ducktape.tests.test import test_logger, TestContext
 
 from ducktape.tests.result import TestResult, IGNORE, PASS, FAIL
@@ -39,7 +42,7 @@ class RunnerClient(object):
     """Run a single test"""
 
     def __init__(self, server_hostname, server_port, test_id,
-                 test_index, logger_name, log_dir, debug, fail_bad_cluster_utilization):
+                 test_index, logger_name, log_dir, debug, fail_bad_cluster_utilization, deflake_num):
         signal.signal(signal.SIGTERM, self._sigterm_handler)  # register a SIGTERM handler
 
         self.serde = SerDe()
@@ -58,9 +61,12 @@ class RunnerClient(object):
         self.test_metadata = ready_reply["test_metadata"]
         self.cluster = ready_reply["cluster"]
 
+        self.deflake_num = deflake_num
+
         # Wait to instantiate the test object until running the test
         self.test = None
         self.test_context = None
+        self.all_services = None
 
     def send(self, event):
         return self.sender.send(event)
@@ -102,60 +108,30 @@ class RunnerClient(object):
 
         start_time = -1
         stop_time = -1
-        test_status = PASS
-        summary = ""
+        test_status = FAIL
+        summary = []
         data = None
+        self.all_services = ServiceRegistry()
+
+        num_runs = 0
 
         try:
-            # Results from this test, as well as logs will be dumped here
-            mkdir_p(TestContext.results_dir(self.test_context, self.test_index))
-            # Instantiate test
-            self.test = self.test_context.cls(self.test_context)
-
-            self.log(logging.DEBUG, "Checking if there are enough nodes...")
-            min_cluster_spec = self.test.min_cluster_spec()
-            os_to_num_nodes = {}
-            for node_spec in min_cluster_spec:
-                if not os_to_num_nodes.get(node_spec.operating_system):
-                    os_to_num_nodes[node_spec.operating_system] = 1
-                else:
-                    os_to_num_nodes[node_spec.operating_system] = os_to_num_nodes[node_spec.operating_system] + 1
-            for (operating_system, node_count) in iteritems(os_to_num_nodes):
-                num_avail = len(list(self.cluster.all().nodes.elements(operating_system=operating_system)))
-                if node_count > num_avail:
-                    raise RuntimeError(
-                        "There are not enough nodes available in the cluster to run this test. "
-                        "Cluster size for %s: %d, Need at least: %d. Services currently registered: %s" %
-                        (operating_system, num_avail, node_count, self.test_context.services))
-
-            # Run the test unit
-            start_time = time.time()
-            self.setup_test()
-
-            data = self.run_test()
-
-            test_status = PASS
-            self.log(logging.INFO, "PASS")
-
-        except BaseException as e:
-            # mark the test as failed before doing anything else
-            test_status = FAIL
-            err_trace = self._exc_msg(e)
-            summary += err_trace
-            self.log(logging.INFO, "FAIL: " + err_trace)
-
+            while test_status == FAIL and num_runs < self.deflake_num:
+                num_runs += 1
+                self.log(logging.INFO, "on run {}/{}".format(num_runs, self.deflake_num))
+                start_time = time.time()
+                test_status, summary, data = self._do_run(num_runs)
         finally:
-            self.teardown_test(teardown_services=not self.session_context.no_teardown, test_status=test_status)
-
             stop_time = time.time()
-
-            if hasattr(self, "services"):
-                service_errors = self.test_context.services.errors()
-                if service_errors:
-                    summary += "\n\n" + service_errors
-
+            if test_status == PASS and num_runs > 1:
+                test_status = FLAKY
+            summary = "".join(summary)
             test_status, summary = self._check_cluster_utilization(test_status, summary)
 
+            if num_runs > 1:
+                # for reporting purposes report all services
+                self.test_context.services = self.all_services
+            # for flaky tests, we report the start and end time of the successful run, and not the whole run period
             result = TestResult(
                 self.test_context,
                 self.test_index,
@@ -176,9 +152,75 @@ class RunnerClient(object):
             # Release test_context resources only after creating the result and finishing logging activity
             # The Sender object uses the same logger, so we postpone closing until after the finished message is sent
             self.test_context.close()
+            self.all_services = None
             self.test_context = None
             self.test = None
 
+    def _do_run(self, num_runs):
+        test_status = FAIL
+        summary = []
+        data = None
+        sid_factory = MultiRunServiceIdFactory(num_runs) if self.deflake_num > 1 else service_id_factory
+        try:
+            # Results from this test, as well as logs will be dumped here
+            mkdir_p(TestContext.results_dir(self.test_context, self.test_index))
+            # Instantiate test
+            self.test = self.test_context.cls(self.test_context)
+            # Check if there are enough nodes
+            self._check_min_cluster_spec()
+            # Run the test unit
+
+            self.setup_test()
+
+            data = self.run_test()
+
+            test_status = PASS
+            self.log(logging.INFO, "{} TEST".format(test_status.to_json()))
+
+        except BaseException as e:
+            # mark the test as failed before doing anything else
+            test_status = FAIL
+            err_trace = self._exc_msg(e)
+            summary.append(err_trace)
+            if num_runs != self.deflake_num:
+                summary.append("~" * max(len(l) for l in err_trace.split('\n')) + "\n")
+            self.log(logging.INFO, "FAIL: " + err_trace)
+
+        finally:
+            for service in self.test_context.services:
+                service.service_id_factory = sid_factory
+                self.all_services.append(service)
+
+            self.teardown_test(teardown_services=not self.session_context.no_teardown, test_status=test_status)
+
+            if hasattr(self.test_context, "services"):
+                service_errors = self.test_context.services.errors()
+                if service_errors:
+                    summary.extend(["\n\n", service_errors])
+
+            # free nodes
+            if self.test:
+                self.log(logging.DEBUG, "Freeing nodes...")
+                self._do_safely(self.test.free_nodes, "Error freeing nodes:")
+            return test_status, summary, data
+
+    def _check_min_cluster_spec(self):
+        self.log(logging.DEBUG, "Checking if there are enough nodes...")
+        min_cluster_spec = self.test.min_cluster_spec()
+        os_to_num_nodes = {}
+        for node_spec in min_cluster_spec:
+            if not os_to_num_nodes.get(node_spec.operating_system):
+                os_to_num_nodes[node_spec.operating_system] = 1
+            else:
+                os_to_num_nodes[node_spec.operating_system] = os_to_num_nodes[node_spec.operating_system] + 1
+        for (operating_system, node_count) in iteritems(os_to_num_nodes):
+            num_avail = len(list(self.cluster.all().nodes.elements(operating_system=operating_system)))
+            if node_count > num_avail:
+                raise RuntimeError(
+                    "There are not enough nodes available in the cluster to run this test. "
+                    "Cluster size for %s: %d, Need at least: %d. Services currently registered: %s" %
+                    (operating_system, num_avail, node_count, self.test_context.services))
+
     def _check_cluster_utilization(self, result, summary):
         """Checks if the number of nodes used by a test is less than the number of
         nodes requested by the test. If this is the case and we wish to fail
@@ -191,7 +233,7 @@ class RunnerClient(object):
             message = "Test requested %d nodes, used only %d" % (total, max_used)
             if self.fail_bad_cluster_utilization:
                 # only check node utilization on test pass
-                if result == PASS:
+                if result == PASS or result == FLAKY:
                     self.log(logging.INFO, "FAIL: " + message)
 
                 result = FAIL
@@ -252,9 +294,6 @@ class RunnerClient(object):
             self.log(logging.DEBUG, "Cleaning up services...")
             self._do_safely(services.clean_all, "Error cleaning services:")
 
-        self.log(logging.DEBUG, "Freeing nodes...")
-        self._do_safely(self.test.free_nodes, "Error freeing nodes:")
-
     def log(self, log_level, msg, *args, **kwargs):
         """Log to the service log and the test log of the current test."""
 
